from binance.lib.utils import check_required_parameter


def fiat_order_history(self, transactionType: int, **kwargs):
    """Get Fiat Deposit/Withdraw History (USER_DATA)

    GET /sapi/v1/fiat/orders

    https://binance-docs.github.io/apidocs/spot/en/#get-fiat-deposit-withdraw-history-user_data

    Args:
      transactionType (int): 0-deposit,1-withdraw
    Keyword Args:
      beginTime (int, optional)
      endTime (int, optional)
      page (int, optional): default 1
      rows (int, optional): default 100, max 500
      recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(transactionType, "transactionType")
    payload = {"transactionType": transactionType, **kwargs}
    return self.sign_request("GET", "/sapi/v1/fiat/orders", payload)


def fiat_payment_history(self, transactionType: int, **kwargs):
    """Get Fiat Payments History (USER_DATA)

    GET /sapi/v1/fiat/payments

    https://binance-docs.github.io/apidocs/spot/en/#get-fiat-payments-history-user_data

    Args:
      transactionType (int): 0-buy,1-sell
    Keyword Args:
      beginTime (int, optional)
      endTime (int, optional)
      page (int, optional): default 1
      rows (int, optional): default 100, max 500
      recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(transactionType, "transactionType")
    payload = {"transactionType": transactionType, **kwargs}
    return self.sign_request("GET", "/sapi/v1/fiat/payments", payload)



from binance.lib.utils import check_required_parameters

"""
  Endpoints that for moving funds between spot and futures accounts.
  It's not for trading on futures.

"""


def futures_transfer(self, asset: str, amount: float, type: int, **kwargs):
    """New Future Account Transfer (USER_DATA)
    Execute transfer between spot account and futures account.

    POST /sapi/v1/futures/transfer

    https://binance-docs.github.io/apidocs/spot/en/#new-future-account-transfer-futures

    Args:
        asset (str): The asset being transferred, e.g. USDT
        amount (float): The amount to be transferred
        type (int): 1: transfer from spot account to USDT-Ⓜ futures account.
              2: transfer from USDT-Ⓜ futures account to spot account.
              3: transfer from spot account to COIN-Ⓜ futures account.
              4: transfer from COIN-Ⓜ futures account to spot account.
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[asset, "asset"], [amount, "amount"], [type, "type"]])

    payload = {"asset": asset, "amount": amount, "type": type, **kwargs}
    return self.sign_request("POST", "/sapi/v1/futures/transfer", payload)


def futures_transfer_history(self, asset: str, startTime, **kwargs):
    """Get Future Account Transaction History List (USER_DATA)

    GET /sapi/v1/futures/transfer

    https://binance-docs.github.io/apidocs/spot/en/#get-future-account-transaction-history-list-user_data

    Args:
        asset (str): The asset being transferred, e.g. USDT
        startTime (int)
    Keyword Args:
        endTime (int, optional)
        current (int, optional): Currently querying page. Start from 1. Default:1
        size (int, optional)
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[asset, "asset"], [startTime, "startTime"]])

    payload = {"asset": asset, "startTime": startTime, **kwargs}
    return self.sign_request("GET", "/sapi/v1/futures/transfer", payload)


def futures_loan_borrow_history(self, **kwargs):
    """Cross-Collateral Borrow History (USER_DATA)

    GET /sapi/v1/futures/loan/borrow/history

    https://binance-docs.github.io/apidocs/spot/en/#cross-collateral-borrow-history-user_data

    Keyword Args:
        coin (str, optional)
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): default 500, max 1000
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/futures/loan/borrow/history", kwargs)


def futures_loan_repay_history(self, **kwargs):
    """Cross-Collateral Repayment History (USER_DATA)

    GET /sapi/v1/futures/loan/repay/history

    https://binance-docs.github.io/apidocs/spot/en/#cross-collateral-repayment-history-user_data

    Keyword Args:
        coin (str, optional)
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): default 500, max 1000
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/futures/loan/repay/history", kwargs)


def futures_loan_wallet(self, **kwargs):
    """Cross-Collateral Wallet (USER_DATA)

    GET /sapi/v2/futures/loan/wallet

    https://binance-docs.github.io/apidocs/spot/en/#cross-collateral-wallet-v2-user_data

    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v2/futures/loan/wallet", kwargs)


def futures_loan_adjust_collateral_history(self, **kwargs):
    """Adjust Cross-Collateral LTV History (USER_DATA)

    GET /sapi/v1/futures/loan/adjustCollateral/history

    https://binance-docs.github.io/apidocs/spot/en/#adjust-cross-collateral-ltv-history-user_data

    Keyword Args:
        loanCoin (str, optional)
        collateralCoin (str, optional)
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): default 500, max 1000
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request(
        "GET", "/sapi/v1/futures/loan/adjustCollateral/history", kwargs
    )


def futures_loan_liquidation_history(self, **kwargs):
    """Cross-Collateral Liquidation History (USER_DATA)

    GET /sapi/v1/futures/loan/liquidationHistory

    https://binance-docs.github.io/apidocs/spot/en/#cross-collateral-liquidation-history-user_data

    Keyword Args:
        loanCoin (str, optional)
        collateralCoin (str, optional)
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): default 500, max 1000
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/futures/loan/liquidationHistory", kwargs)


def futures_loan_interest_history(self, **kwargs):
    """Cross-Collateral Interest History (USER_DATA)

    GET /sapi/v1/futures/loan/interestHistory

    https://binance-docs.github.io/apidocs/spot/en/#cross-collateral-interest-history-user_data

    Keyword Args:
        collateralCoin (str, optional)
        startTime (int, optional)
        endTime (int, optional)
        current (int, optional): Currently querying page. Start from 1. Default:1
        limit (int, optional): default 500, max 1000
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/futures/loan/interestHistory", kwargs)



def rebate_spot_history(self, **kwargs):
    """Get Spot Rebate History Records (USER_DATA)

    GET /sapi/v1/rebate/taxQuery

    https://binance-docs.github.io/apidocs/spot/en/#get-spot-rebate-history-records-user_data

    Keyword Args:
      startTime (int, optional)
      endTime (int, optional)
      page (int, optional): default 1
      recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/rebate/taxQuery", kwargs)



from binance.websocket.websocket_client import BinanceWebsocketClient


class SpotWebsocketClient(BinanceWebsocketClient):
    def __init__(self, stream_url="wss://stream.binance.com:9443"):
        super().__init__(stream_url)

    def agg_trade(self, symbol: str, id: int, callback, **kwargs):
        """Aggregate Trade Streams

        The Aggregate Trade Streams push trade information that is aggregated for a single taker order.

        Stream Name: <symbol>@aggTrade

        Update Speed: Real-time
        """
        if isinstance(symbol, list):
            symbol = ["{}@aggTrade".format(x.lower()) for x in symbol]
        else:
            symbol = "{}@aggTrade".format(symbol.lower())

        self.live_subscribe(symbol, id, callback, **kwargs)

    def trade(self, symbol: str, id: int, callback, **kwargs):
        """Trade Streams

        The Trade Streams push raw trade information; each trade has a unique buyer and seller.

        Stream Name: <symbol>@trade

        Update Speed: Real-time
        """
        if isinstance(symbol, list):
            symbol = ["{}@trade".format(x.lower()) for x in symbol]
        else:
            symbol = "{}@trade".format(symbol.lower())

        self.live_subscribe(symbol, id, callback, **kwargs)

    def kline(self, symbol: str, id: int, interval: str, callback, **kwargs):
        """Kline/Candlestick Streams

        The Kline/Candlestick Stream push updates to the current klines/candlestick every second.

        Stream Name: <symbol>@kline_<interval>

        interval:
        m -> minutes; h -> hours; d -> days; w -> weeks; M -> months

        - 1m
        - 3m
        - 5m
        - 15m
        - 30m
        - 1h
        - 2h
        - 4h
        - 6h
        - 8h
        - 12h
        - 1d
        - 3d
        - 1w
        - 1M

        Update Speed: 2000ms
        """
        if isinstance(symbol, list):
            symbol = ["{}@kline_{}".format(x.lower(), interval) for x in symbol]
        else:
            symbol = "{}@kline_{}".format(symbol.lower(), interval)

        self.live_subscribe(symbol, id, callback, **kwargs)

    def mini_ticker(self, id: int, callback, symbol=None, **kwargs):
        """Individual symbol or all symbols mini ticker

        24hr rolling window mini-ticker statistics.
        These are NOT the statistics of the UTC day, but a 24hr rolling window for the previous 24hrs

        Stream Name: <symbol>@miniTicker or
        Stream Name: !miniTicker@arr

        Update Speed: 1000ms
        """

        if symbol is None:
            self.live_subscribe("!miniTicker@arr", id, callback, **kwargs)
        else:
            if isinstance(symbol, list):
                symbol = ["{}@miniTicker".format(x.lower()) for x in symbol]
            else:
                symbol = "{}@miniTicker".format(symbol.lower())

            self.live_subscribe(symbol, id, callback, **kwargs)

    def ticker(self, id: int, callback, symbol=None, **kwargs):
        """Individual symbol or all symbols ticker

        24hr rolling window ticker statistics for a single symbol.
        These are NOT the statistics of the UTC day, but a 24hr rolling window for the previous 24hrs.

        Stream Name: <symbol>@ticker or
        Stream Name: !ticker@arr

        Update Speed: 1000ms
        """

        if symbol is None:
            self.live_subscribe("!ticker@arr", id, callback, **kwargs)
        else:
            if isinstance(symbol, list):
                symbol = ["{}@ticker".format(x.lower()) for x in symbol]
            else:
                symbol = "{}@ticker".format(symbol.lower())

            self.live_subscribe(symbol, id, callback, **kwargs)

    def book_ticker(self, id: int, callback, symbol=None, **kwargs):
        """Individual symbol or all book ticker

        Pushes any update to the best bid or ask's price or quantity in real-time for a specified symbol.

        Stream Name: <symbol>@bookTicker or
        Stream Name: !bookTicker

        Update Speed: realtime
        """

        if symbol is None:
            self.live_subscribe("!bookTicker", id, callback, **kwargs)
        else:
            if isinstance(symbol, list):
                symbol = ["{}@bookTicker".format(x.lower()) for x in symbol]
            else:
                symbol = "{}@bookTicker".format(symbol.lower())

            self.live_subscribe(symbol, id, callback, **kwargs)

    def partial_book_depth(
        self, symbol: str, id: int, level, speed, callback, **kwargs
    ):
        """Partial Book Depth Streams

        Top bids and asks, Valid are 5, 10, or 20.

        Stream Names: <symbol>@depth<levels> OR <symbol>@depth<levels>@100ms.

        Update Speed: 1000ms or 100ms
        """

        if isinstance(symbol, list):
            symbol = ["{}@depth{}@{}ms".format(x.lower(), level, speed) for x in symbol]
        else:
            symbol = "{}@depth{}@{}ms".format(symbol.lower(), level, speed)

        self.live_subscribe(symbol, id, callback, **kwargs)

    def rolling_window_ticker(
        self, symbol: str, windowSize: str, id: int, callback, **kwargs
    ):
        """Rolling window ticker statistics for a single symbol, computed over multiple windows.

        Stream Name: <symbol>@ticker_<window_size>

        Window Sizes: 1h,4h

        Update Speed: 1000ms

        Note: This stream is different from the <symbol>@ticker stream. The open time "O" always starts on a minute, while the closing time "C" is the current time of the update. As such, the effective window might be up to 59999ms wider that <window_size>.
        """

        if isinstance(symbol, list):
            symbol = ["{}@ticker_{}".format(x.lower(), windowSize) for x in symbol]
        else:
            symbol = "{}@ticker_{}".format(symbol.lower(), windowSize)

        self.live_subscribe(symbol, id, callback, **kwargs)

    def rolling_window_ticker_all_symbols(
        self, windowSize: str, id: int, callback, **kwargs
    ):
        """All Market Rolling Window Statistics Streams

        Rolling window ticker statistics for all market symbols, computed over multiple windows. Note that only tickers that have changed will be present in the array.

        Stream Name: !ticker_<window-size>@arr

        Window Size: 1h,4h

        Update Speed: 1000ms
        """
        self.live_subscribe("!ticker_{}@arr".format(windowSize), id, callback, **kwargs)

    def diff_book_depth(self, symbol: str, id: int, speed, callback, **kwargs):
        """Diff. Depth Stream

        Stream Name: <symbol>@depth OR <symbol>@depth@100ms

        Update Speed: 1000ms or 100ms

        Order book price and quantity depth updates used to locally manage an order book.
        """
        if isinstance(symbol, list):
            symbol = ["{}@depth@{}ms".format(x.lower(), speed) for x in symbol]
        else:
            symbol = "{}@depth@{}ms".format(symbol.lower(), speed)

        self.live_subscribe(symbol, id, callback, **kwargs)

    def user_data(self, listen_key: str, id: int, callback, **kwargs):
        """Listen to user data by using the provided listen_key"""
        self.live_subscribe(listen_key, id, callback, **kwargs)



from binance.lib.utils import check_required_parameters, check_required_parameter


def loan_history(self, asset: str, **kwargs):
    """Get Crypto Loans Income History (USER_DATA)

    GET /sapi/v1/loan/income

    https://binance-docs.github.io/apidocs/spot/en/#get-crypto-loans-income-history-user_data

    Args:
      asset (str)
    Keyword Args:
      type (str, optional): All types will be returned by default.
                            borrowIn, collateralSpent, repayAmount, collateralReturn (collateral return after repayment),
                            addCollateral, removeCollateral, collateralReturnAfterLiquidation
      startTime (int, optional)
      endTime (int, optional)
      limit (int, optional): default 20, max 100
      recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(asset, "asset")

    payload = {"asset": asset, **kwargs}
    return self.sign_request("GET", "/sapi/v1/loan/income", payload)


def loan_borrow(self, loanCoin: str, collateralCoin: str, loanTerm: int, **kwargs):
    """Crypto Loan Borrow (TRADE)

    POST /sapi/v1/loan/borrow

    https://binance-docs.github.io/apidocs/spot/en/#borrow-crypto-loan-borrow-trade

    Args:
      loanCoin (str)
      collateralCoin (str)
      loanTerm (int): 7/14/30/90/180 days
    Keyword Args:
      loanAmount (float, optional): Mandatory when collateralAmount is empty
      collateralAmount (float, optional): Mandatory when loanAmount is empty
      recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters(
        [
            [loanCoin, "loanCoin"],
            [collateralCoin, "collateralCoin"],
            [loanTerm, "loanTerm"],
        ]
    )

    payload = {
        "loanCoin": loanCoin,
        "collateralCoin": collateralCoin,
        "loanTerm": loanTerm,
        **kwargs,
    }
    return self.sign_request("POST", "/sapi/v1/loan/borrow", payload)


def loan_borrow_history(self, **kwargs):
    """Get Loan Borrow History (USER_DATA)

    GET /sapi/v1/loan/borrow/history

    https://binance-docs.github.io/apidocs/spot/en/#borrow-get-loan-borrow-history-user_data

    Keyword Args:
      orderId (int, optional): orderId in POST /sapi/v1/loan/borrow
      loanCoin (str, optional)
      collateralCoin (str, optional)
      startTime (int, optional)
      endTime (int, optional)
      current (int, optional): Current querying page. Start from 1; default: 1; max: 1000.
      limit (int, optional): Default: 10; max: 100
      recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/loan/borrow/history", kwargs)


def loan_ongoing_orders(self, **kwargs):
    """Get Loan Ongoing Orders (USER_DATA)

    GET /sapi/v1/loan/ongoing/orders

    https://binance-docs.github.io/apidocs/spot/en/#borrow-get-loan-ongoing-orders-user_data

    Keyword Args:
      orderId (int, optional): orderId in POST /sapi/v1/loan/borrow
      loanCoin (str, optional)
      collateralCoin (str, optional)
      current (int, optional): Current querying page. Start from 1; default: 1; max: 1000
      limit (int, optional): Default: 10; max: 100
      recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/loan/ongoing/orders", kwargs)


def loan_repay(self, orderId: int, amount: float, **kwargs):
    """Crypto Loan Repay (TRADE)

    POST /sapi/v1/loan/repay

    https://binance-docs.github.io/apidocs/spot/en/#repay-crypto-loan-repay-trade

    Args:
      orderId (int)
      amount (float)
    Keyword Args:
      type (int, optional): Default: 1. 1 for "repay with borrowed coin"; 2 for "repay with collateral"
      collateralReturn (boolean, optional): Default: TRUE. TRUE: Return extra collateral to spot account; FALSE: Keep extra collateral in the order.
      recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[orderId, "orderId"], [amount, "amount"]])

    payload = {"orderId": orderId, "amount": amount, **kwargs}
    return self.sign_request("POST", "/sapi/v1/loan/repay", payload)


def loan_repay_history(self, **kwargs):
    """Get Loan Repayment History (USER_DATA)

    GET /sapi/v1/loan/repay/history

    https://binance-docs.github.io/apidocs/spot/en/#repay-get-loan-repayment-history-user_data

    Keyword Args:
      orderId (int, optional)
      loanCoin (str, optional)
      collateralCoin (str, optional)
      startTime (int, optional)
      endTime (int, optional)
      current (int, optional): Current querying page. Start from 1; default: 1; max: 1000.
      limit (int, optional): Default: 10; max: 100
      recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/loan/repay/history", kwargs)


def loan_adjust_ltv(self, orderId: int, amount: float, direction: str, **kwargs):
    """Crypto Loan Adjust LTV (TRADE)

    POST /sapi/v1/loan/adjust/ltv

    https://binance-docs.github.io/apidocs/spot/en/#adjust-ltv-crypto-loan-adjust-ltv-trade

    Args:
      orderId (int)
      amount (float)
      direction (str): "ADDITIONAL", "REDUCED"
    Keyword Args:
      recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters(
        [[orderId, "orderId"], [amount, "amount"], [direction, "direction"]]
    )

    payload = {"orderId": orderId, "amount": amount, "direction": direction, **kwargs}
    return self.sign_request("POST", "/sapi/v1/loan/adjust/ltv", payload)


def loan_adjust_ltv_history(self, **kwargs):
    """Get Loan LTV Adjustment History (USER_DATA)

    GET /sapi/v1/loan/ltv/adjustment/history

    https://binance-docs.github.io/apidocs/spot/en/#adjust-ltv-get-loan-ltv-adjustment-history-user_data

    Keyword Args:
      orderId (int, optional)
      loanCoin (str, optional)
      collateralCoin (str, optional)
      startTime (int, optional)
      endTime (int, optional)
      current (int, optional): Current querying page. Start from 1; default: 1; max: 1000
      limit (int, optional): Default: 10; max: 100
      recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/loan/ltv/adjustment/history", kwargs)



def pay_history(self, **kwargs):
    """Get Pay Trade History (USER_DATA)

    GET /sapi/v1/pay/transactions

    https://binance-docs.github.io/apidocs/spot/en/#get-pay-trade-history-user_data

    Keyword Args:
      startTime (int, optional)
      endTime (int, optional)
      limit (int, optional): default 100, max 100
      recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/pay/transactions", kwargs)



from binance.lib.utils import check_required_parameter


def c2c_trade_history(self, tradeType: str, **kwargs):
    """Get C2C Trade History (USER_DATA)

    GET /sapi/v1/c2c/orderMatch/listUserOrderHistory

    https://binance-docs.github.io/apidocs/spot/en/#get-c2c-trade-history-user_data

    Args:
      tradeType (str): BUY, SELL
    Keyword Args:
      startTimestamp (int, optional)
      endTimestamp (int, optional)
      page (int, optional): default 1
      rows (int, optional): default 100, max 100
      recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(tradeType, "tradeType")

    payload = {"tradeType": tradeType, **kwargs}
    return self.sign_request(
        "GET", "/sapi/v1/c2c/orderMatch/listUserOrderHistory", payload
    )



import hmac
import hashlib
from base64 import b64encode
from Crypto.PublicKey import RSA
from Crypto.Hash import SHA256
from Crypto.Signature import pkcs1_15


def hmac_hashing(api_secret, payload):
    m = hmac.new(api_secret.encode("utf-8"), payload.encode("utf-8"), hashlib.sha256)
    return m.hexdigest()


def rsa_signature(private_key, payload, private_key_pass=None):
    private_key = RSA.import_key(private_key, passphrase=private_key_pass)
    h = SHA256.new(payload.encode("utf-8"))
    signature = pkcs1_15.new(private_key).sign(h)
    return b64encode(signature)
